Package: readapra
Title: Download and Tidy Data from the Australian Prudential Regulation
        Authority
Version: 0.2.3
Authors@R: 
    person(given = "Jarrod", 
           family = "van der Wal", 
           email = "jarvanderwal@gmail.com",
           role = c("aut", "cre", "cph")
           )
Description: Download the latest data from the Australian Prudential Regulation
    Authority <https://www.apra.gov.au/> and import it into R as a tidy data 
    frame. 
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Suggests: openxlsx, readr, testthat (>= 3.0.0), webmockr, withr
Config/testthat/edition: 3
URL: https://github.com/javanderwal/readapra,
        https://javanderwal.github.io/readapra/
BugReports: https://github.com/javanderwal/readapra/issues
Imports: cli (>= 3.0.0), dplyr (>= 1.0.0), httr (>= 1.0.0), janitor,
        lubridate (>= 1.0.0), purrr (>= 1.0.0), rlang (>= 1.0.0), rvest
        (>= 1.0.0), stringdist, stringr (>= 1.0.0), tibble (>= 2.0.0),
        tidyr (>= 1.0.0), tidyselect (>= 1.0.0), tidyxl (>= 1.0.0),
        utils, xml2
Depends: R (>= 3.5)
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-03 00:04:39 UTC; jarva
Author: Jarrod van der Wal [aut, cre, cph]
Maintainer: Jarrod van der Wal <jarvanderwal@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-03 05:10:02 UTC
Built: R 4.6.0; ; 2026-05-03 09:03:32 UTC; unix
