#' Statistical Publications Produced by APRA
#'
#' This dataset contains a record of all statistical publications produced by
#' APRA that are available in `readapra`.
#'
#' @format A data frame with 7 rows and 3 variables:
#' \describe{
#'  \item{stat_pub_name}{the full name of the statistical publication.}
#'  \item{stat_pub_acronym}{the acronym of the statistical publication.}
#'  \item{cur_hist}{available current and historical versions of the statistical publication.}
#' }
#'
"apra_stat_pubs"
